﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

namespace TestProject
{
    /// <summary>
    /// 고정 이미지
    /// </summary>
    public class FixedImage : Image
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 오프셋 포인트 - OffsetPoint

        /// <summary>
        /// 오프셋 포인트
        /// </summary>
        private Point OffsetPoint { get; set; }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - FixedImage()

        /// <summary>
        /// 생성자
        /// </summary>
        public FixedImage()
        {
            LayoutUpdated += Image_LayoutUpdated;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected
        //////////////////////////////////////////////////////////////////////////////// Method

        #region 렌더링시 처리하기 - OnRender(drawingContext)

        /// <summary>
        /// 렌더링시 처리하기
        /// </summary>
        /// <param name="drawingContext">드로잉 컨텍스트</param>
        protected override void OnRender(DrawingContext drawingContext)
        {
            OffsetPoint = GetOffsetPoint();

            drawingContext.PushTransform(new TranslateTransform(OffsetPoint.X, OffsetPoint.Y));

            base.OnRender(drawingContext);
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 이미지 레이아웃 업데이트시 처리하기 - Image_LayoutUpdated(sender, e)

        /// <summary>
        /// 이미지 레이아웃 업데이트시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Image_LayoutUpdated(object sender, EventArgs e)
        {
            Point currentOffsetPoint = GetOffsetPoint();

            double threshold = 0.001; // 1/100 픽셀
            double deltaX;
            double deltaY;

            deltaX = currentOffsetPoint.X - OffsetPoint.X;
            deltaY = currentOffsetPoint.Y - OffsetPoint.Y;

            if((((deltaX < threshold) && (deltaX > -threshold)) == false) || (((deltaY < threshold) && (deltaY > -threshold)) == false))
            {
                InvalidateVisual();
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 오프셋 포인트 구하기 - GetOffsetPoint()

        /// <summary>
        /// 오프셋 포인트 구하기
        /// </summary>
        /// <returns>오프셋 포인트</returns>
        private Point GetOffsetPoint()
        {
            Point offsetPoint = new Point();

            PresentationSource source = PresentationSource.FromVisual(this);

            if(source != null)
            {
                Visual rootVisual = source.RootVisual;

                offsetPoint = this.TransformToAncestor(rootVisual).Transform(new Point(0, 0));

                offsetPoint = source.CompositionTarget.TransformToDevice.Transform(offsetPoint);

                offsetPoint.X = Math.Round(offsetPoint.X);
                offsetPoint.Y = Math.Round(offsetPoint.Y);

                offsetPoint = source.CompositionTarget.TransformFromDevice.Transform(offsetPoint);

                offsetPoint = rootVisual.TransformToDescendant(this).Transform(offsetPoint);
            }

            return offsetPoint;
        }

        #endregion
    }
}